@@ -42,6 +42,15 @@
 using namespace std;
 using namespace CEC;
 
+CLibCEC::CLibCEC(const char *strDeviceName, cec_device_type_list types) :
+    m_iStartTime(GetTimeMs()),
+    m_iCurrentButton(CEC_USER_CONTROL_CODE_UNKNOWN),
+    m_buttontime(0)
+{
+  m_comm = new CAdapterCommunication(this);
+  m_cec = new CCECProcessor(this, m_comm, strDeviceName, types);
+}
+
 CLibCEC::CLibCEC(const char *strDeviceName, cec_logical_address iLogicalAddress /* = CECDEVICE_PLAYBACKDEVICE1 */, uint16_t iPhysicalAddress /* = CEC_DEFAULT_PHYSICAL_ADDRESS */) :
     m_iStartTime(GetTimeMs()),
     m_iCurrentButton(CEC_USER_CONTROL_CODE_UNKNOWN),
@@ -167,19 +176,41 @@ bool CLibCEC::StandbyDevices(cec_logical_address address /* = CECDEVICE_BROADCAS
   return m_cec && address >= CECDEVICE_TV && address <= CECDEVICE_BROADCAST ? m_cec->m_busDevices[(uint8_t)address]->Standby() : false;
 }
 
+bool CLibCEC::SetActiveSource(cec_device_type type /* = CEC_DEVICE_TYPE_RESERVED */)
+{
+  return m_cec ? m_cec->SetActiveSource(type) : false;
+}
+
 bool CLibCEC::SetActiveView(void)
 {
   return m_cec ? m_cec->SetActiveView() : false;
 }
 
+bool CLibCEC::SetDeckControlMode(cec_deck_control_mode mode, bool bSendUpdate /* = true */)
+{
+  return m_cec ? m_cec->SetDeckControlMode(mode, bSendUpdate) : false;
+}
+
+bool CLibCEC::SetDeckInfo(cec_deck_info info, bool bSendUpdate /* = true */)
+{
+  return m_cec ? m_cec->SetDeckInfo(info, bSendUpdate) : false;
+}
+
 bool CLibCEC::SetInactiveView(void)
 {
   return m_cec ? m_cec->SetInactiveView() : false;
 }
 
+bool CLibCEC::SetMenuState(cec_menu_state state, bool bSendUpdate /* = true */)
+{
+  return m_cec ? m_cec->SetMenuState(state, bSendUpdate) : false;
+}
+
 bool CLibCEC::SetOSDString(cec_logical_address iLogicalAddress, cec_display_control duration, const char *strMessage)
 {
-  return m_cec && iLogicalAddress >= CECDEVICE_TV && iLogicalAddress <= CECDEVICE_BROADCAST ? m_cec->m_busDevices[(uint8_t)iLogicalAddress]->SetOSDString(duration, strMessage) : false;
+  return m_cec && iLogicalAddress >= CECDEVICE_TV && iLogicalAddress <= CECDEVICE_BROADCAST ?
+      m_cec->m_busDevices[m_cec->GetLogicalAddress()]->TransmitOSDString(iLogicalAddress, duration, strMessage) :
+      false;
 }
 
 bool CLibCEC::SwitchMonitoring(bool bEnable)
@@ -215,6 +246,13 @@ cec_power_status CLibCEC::GetDevicePowerStatus(cec_logical_address iAddress)
   return CEC_POWER_STATUS_UNKNOWN;
 }
 
+bool CLibCEC::PollDevice(cec_logical_address iAddress)
+{
+  if (m_cec && iAddress >= CECDEVICE_TV && iAddress < CECDEVICE_BROADCAST)
+    return m_cec->PollDevice(iAddress);
+  return false;
+}
+
 void CLibCEC::AddLog(cec_log_level level, const string &strMessage)
 {
   if (m_cec)
@@ -289,6 +327,11 @@ void * CECCreate(const char *strDeviceName, CEC::cec_logical_address iLogicalAdd
   return static_cast< void* > (new CLibCEC(strDeviceName, iLogicalAddress, iPhysicalAddress));
 }
 
+void * CECInit(const char *strDeviceName, CEC::cec_device_type_list types)
+{
+  return static_cast< void* > (new CLibCEC(strDeviceName, types));
+}
+
 void CECDestroy(CEC::ICECAdapter *instance)
 {
   CLibCEC *lib = static_cast< CLibCEC* > (instance);
